<?php

/**
 * Question Model
 *
 *
 * @link       https://entusiastics.com
 * @since      1.0.0
 *
 * @package    Entusiastics_Quiz_Game
 * @subpackage Entusiastics_Quiz_Game/admin/models
 */

if( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Question {

	/**
	 * Get Question by Quiz ID
	 *
	 * @return 	array Array of next question and this choices
	 */
	public function getQuestionByQuizId($quiz_id) {

		global $wpdb;

		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." WHERE quiz_id=%d ORDER BY sort_order", $quiz_id);
		$questions = $wpdb->get_results($sql);

		return $questions;

	} // end getQuestionByQuizId()

	public function getFirstQuizQuestion($quiz_id){
		global $wpdb;

		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." WHERE quiz_id=%d ORDER BY sort_order, id LIMIT 1", $quiz_id);
		$question = $wpdb->get_row($sql);

		return $question;
	}


	public function getQuestionById($question_id) {

		global $wpdb;

		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." WHERE id=%d", $question_id);
		$question = $wpdb->get_row($sql);

		return $question;
	} // end getQuestionById()


	public function shiftQuestions($move, $new_order, $quiz_id) {

		global $wpdb;

		$result = $wpdb->query($wpdb->prepare("UPDATE ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." SET sort_order=sort_order+1 WHERE id!=%d AND sort_order=%d AND quiz_id=%d", $move, $new_order, $quiz_id));

		return $result;

	} // end getQuestionById()
	
	
	function add($vars, $quiz_id) {
		
		global $wpdb;
		
		$question = sanitize_text_field($vars['question']);
		$question_aclaration = strip_tags($vars['question_aclaration']);
		$rango = intval(@$vars['rango']);
		$autocontinue = empty($vars['autocontinue']) ? 0 : 1;
		$video_url = $vars['video_url'];
		$video_id = intval($vars['video_id']);
		$video_type = $vars['video_type'];
		$video_ends = empty($vars['video_ends']) ? 0 : 1;
		$image_url = $vars['image_url'];
		$image_id = $vars['image_id'];
		$symbol = $vars['symbol'];

		if(!in_array($vars['qtype'], array('radio', 'checkbox', 'text'))) $vars['qtype'] = 'radio';

		// sort order
		$sql = $wpdb->prepare("SELECT MAX(sort_order) FROM ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." WHERE quiz_id=%d", $quiz_id);
		$sort_order = $wpdb->get_var($sql);
		$sort_order++;	 
		
		$result = $wpdb->query($wpdb->prepare("INSERT INTO ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." SET quiz_id=%d, question=%s, qtype=%s, rango=%d, question_aclaration=%s, autocontinue=%d, sort_order=%d, video_url=%s, video_id=%s, video_type=%s, video_ends=%d, image_url=%s, image_id=%s, symbol=%s", $quiz_id, $question, $vars['qtype'], $rango, $question_aclaration, $autocontinue, $sort_order, $video_url, $video_id, $video_type, $video_ends, $image_url, $image_id, $symbol));
		
		if($result === false) throw new Exception(__('DB Error', 'entusiastics-quiz-game'));

		return $wpdb->insert_id;	

	} // end add
	
	function save($vars, $id) {

		global $wpdb;
		
		$question = sanitize_text_field($vars['question']);
		$question_aclaration = strip_tags($vars['question_aclaration']);
		$autocontinue = empty($vars['autocontinue']) ? 0 : 1;
		$video_url = $vars['video_url'];
		$video_id = intval($vars['video_id']);
		$video_type = $vars['video_type'];
		$video_ends = empty($vars['video_ends']) ? 0 : 1;
		$image_url = $vars['image_url'];
		$image_id = $vars['image_id'];
		$symbol = $vars['symbol'];

		if(!in_array($vars['qtype'], array('radio', 'checkbox', 'text'))) $vars['qtype'] = 'radio';	

		$vars['question'] = wp_kses_post($vars['question']);
		
		$result = $wpdb->query($wpdb->prepare("UPDATE ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." SET question=%s, qtype=%s, question_aclaration=%s, autocontinue=%d, video_url=%s, video_id=%s, video_type=%s, video_ends=%d, image_url=%s, image_id=%s, symbol=%s WHERE id=%d", $question, $vars['qtype'], $question_aclaration, $autocontinue, $video_url, $video_id, $video_type, $video_ends, $image_url, $image_id, $symbol, $id));
		
		if($result === false) throw new Exception(__('DB Error', 'entusiastics-quiz-game'));

		return true;	
	}
	
	function delete($id) {

		global $wpdb;
		
		$id = intval($id);
		
		// delete choices		
		$wpdb->query($wpdb->prepare("DELETE FROM ".ENTUSIASTICS_QUIZ_GAME_CHOICES." WHERE question_id=%d", $id));
		
		// delete question
		$result = $wpdb->query($wpdb->prepare("DELETE FROM ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." WHERE id=%d", $id));
		
		if($result === false) throw new Exception(__('DB Error', 'entusiastics-quiz-game'));

		return true;	
	}
	
	// saves the choices on a question
	function save_choices($vars, $id) {
		global $wpdb;
		
		$id = intval($id);
		
		// edit/delete existing choices
		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_CHOICES." WHERE question_id=%d ORDER BY id ", $id);
		$choices = $wpdb->get_results($sql);
		
		foreach($choices as $choice) {

			if(!empty($_POST['dels']) and in_array($choice->id, $_POST['dels'])) {
				$wpdb->query($wpdb->prepare("DELETE FROM ".ENTUSIASTICS_QUIZ_GAME_CHOICES." WHERE id=%d", $choice->id));
				continue;
			}
			
			if(!current_user_can('unfiltered_html')) {
				$_POST['answer'.$choice->id] = strip_tags($_POST['answer'.$choice->id]);				
			}
			
			$_POST['goto'.$choice->id] = sanitize_text_field($_POST['goto'.$choice->id]);

			$_POST['profile_id'.$choice->id] = sanitize_text_field($_POST['profile_id'.$choice->id]);

			$_POST['choice_video_id'.$choice->id] = sanitize_text_field($_POST['choice_video_id'.$choice->id]);
			$_POST['choice_video_url'.$choice->id] = sanitize_text_field($_POST['choice_video_url'.$choice->id]);
			$_POST['choice_video_type'.$choice->id] = sanitize_text_field($_POST['choice_video_type'.$choice->id]);
			
			// else update
			$wpdb->query($wpdb->prepare("UPDATE ".ENTUSIASTICS_QUIZ_GAME_CHOICES." SET
				choice=%s, goto=%s, profile_id=%d, video_id=%d, video_url=%s, video_type=%s WHERE id=%d", 
				$_POST['answer'.$choice->id], $_POST['goto'.$choice->id], $_POST['profile_id'.$choice->id],$_POST['choice_video_id'.$choice->id],$_POST['choice_video_url'.$choice->id], $_POST['choice_video_type'.$choice->id], $choice->id));
		}	
		
		// add new choices
		$counter = 1;
		foreach($_POST['answers'] as $answer) {

			$counter++;
			if($answer === '') continue;
			
			$answer = strip_tags($answer);
			
			// now insert the choice
			$wpdb->query( $wpdb->prepare("INSERT INTO ".ENTUSIASTICS_QUIZ_GAME_CHOICES." SET question_id=%d, choice=%s, goto=%s, quiz_id=%d, profile_id=%d", $id, $answer, $_POST['goto'][($counter-2)], intval($_POST['quiz_id']), intval($_POST['profile_id'])) );
		}
	} // end save_choices

	// displays the question contents
	function display_question($question) {
		$content = empty($question->question) ? '' : stripslashes($question->question);		
		$content = wpautop($content);
		$content = $GLOBALS['wp_embed']->run_shortcode($content);
		$content = do_shortcode($content);		
		return $content;
	}

  // displays the possible choices on a question
	function display_choices($question, $choices) {

		$autocontinue = $output = '';

		if($question->qtype == 'radio' and $question->autocontinue == 0) {
			$autocontinue = "onclick=\"entusiasticsQuizGame.goon(".$question->quiz_id.", '".admin_url('admin-ajax.php')."');\"";
			$output .= '<!--hide_go_ahead-->';
		}  	   
		
		switch($question->qtype) {
			case 'text':
			return "<div class='entusiastics-quiz-choice'><textarea class='entusiastics-quiz-frontend' required='required' name='answer' rows='5' cols='80'></textarea></div>";
			break;
			case 'checkbox':
			$type = $question->qtype;
			$name = "answers[]";				
			
			foreach($choices as $choice) {
				$choice_text = stripslashes($choice->choice);
				$choice_text = do_shortcode($choice_text);
				
				$output .= "<div class='entusiastics-quiz-choice'><label class='entusiastics-quiz-label'><input class='entusiastics-quiz-frontend entusiastics-quiz-$type' data-profile='".$choice->profile_id."' type='$type' name='$name' value='".$choice->id."' $autocontinue>$choice_text</label></div>";

				$counter++;
			}
			
			return $output;
			break;
			break;
			case 'radio':
			$alphabet = array('A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z');
			$numbers = [];
			foreach (range(0, 100, 1) as $rango) {
				$numbers = $rango;
			}
			$counter = 0;
			$current_symbol = ($question->symbol == 'letters') ?  $alphabet : $numbers;
			$type = $question->qtype;
			$name = "answer";				
			
			foreach($choices as $choice) {
				$choice_text = stripslashes($choice->choice);
				$choice_text = do_shortcode($choice_text);

				$_profile = new Profile();
				$profile = $_profile->getProfileById($choice->profile_id);
				
				$output .= "<div class='entusiastics-quiz-choice'><label class='entusiastics-quiz-label'><span class='entusiastic-choice-symbol'>".$current_symbol[$counter]."</span><input class='entusiastics-quiz-frontend entusiastics-quiz-$type' data-title='".$choice_text."' data-connotation='".$profile->connotation."' data-profile_name='".$profile->name."' data-profile='".$choice->profile_id."' type='$type' name='$name' value='".$choice->id."' $autocontinue> $choice_text</label></div>";

				$counter++;
			}
			
			return $output;
			break;
		}
  } // end display_choices
  
  // gets the next question in a quiz, depending on the given answer
  function next($question, $answer) {

  	global $wpdb; 	
  	
	// select answer(s)
  	$goto = array();
  	$answer_ids = array(0);

  	if(is_array($answer)) {

  		foreach($answer as $ans) {
  			if(!empty($ans)) $answer_ids[] = $ans;
  		}
  	} 
  	else {

  		if($question->qtype == 'text') {
  			$sql = $wpdb->prepare("SELECT id FROM ".ENTUSIASTICS_QUIZ_GAME_CHOICES."
  				WHERE question_id=%d AND choice LIKE %s", $question->id, $answer);
  			$answer = $wpdb->get_var($sql);	
  		} 
  		
			if(!empty($answer)) $answer_ids[] = $answer; // radio buttons and text areas
		} 
		
		$answer_ids = entusiastics_quiz_game_int_array($answer_ids);
		if(empty($answer_ids)) $answer_ids = array(0);
		
		// select the choices selected by the user		
		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_CHOICES." WHERE question_id=%d AND id IN (".implode(",", $answer_ids).") ", $question->id);
		$choices = $wpdb->get_results($sql);	
		
		foreach($choices as $choice) {
			if(isset($goto[$choice->goto])) $goto[$choice->goto]++;
			else $goto[$choice->goto] = 1;			
		}
		
		// now sort goto to figure out what's the top goto selection	
		arsort($goto);		
		$goto = array_flip($goto);
		$key = array_shift($goto);
		
		// let's treat textareas in different way. If answer is not found, let's not finalize the quiz but go to next
		if($question->qtype == 'text' and empty($key)) $key = 'next';
		
		// echo $key.'x'; 
		if(empty($key) or $key == 'finalize') return false;
		
		if($key == 'next') {

			// select next question by sort_order
			$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." 
				WHERE quiz_id=%d AND sort_order > %d ORDER BY sort_order LIMIT 1", $question->quiz_id, $question->sort_order);
			$question = $wpdb->get_row($sql);

			return $question;	
		}
		
		if(is_numeric($key)) {

			$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." 
				WHERE quiz_id=%d AND id=%d LIMIT 1", $question->quiz_id, $key);
			$question = $wpdb->get_row($sql);

			return $question;	
		}
		
	  	// just in case
		return false;

	} // end next()

} 