<?php

/**
 * Quiz Model
 *
 *
 * @link       https://entusiastics.com
 * @since      1.0.0
 *
 * @package    Entusiastics_Quiz_Game
 * @subpackage Entusiastics_Quiz_Game/admin/models
 */

if( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Quiz {

	public function getQuizById($quiz_id) {

		global $wpdb;

		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_QUIZZES." WHERE id=%d", $quiz_id);
		$quiz = $wpdb->get_row($sql);

		return $quiz;

	} // end getQuizById()

	public function getQuizzesByCategoryId($category_id) {
		global $wpdb;

		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_QUIZZES." WHERE category_id=%d", $category_id);
		$quiz = $wpdb->get_row($sql);

		return $quiz;
	} // end getQuizById()getQuizzesByCategoryId


	public function getRelatedCategoryQuizzes($quiz_id, $category_id) {

		global $wpdb;

		$order_by = "created_at";

		$sql = $wpdb->prepare("SELECT *	FROM ".ENTUSIASTICS_QUIZ_GAME_QUIZZES." WHERE id <> %d ORDER BY %s DESC", $quiz_id, $order_by);
		$quizzes = $wpdb->get_results($sql);

		// now select all posts that have entusiastics-quiz-game shortcode in them
		$posts = $wpdb->get_results("SELECT * FROM {$wpdb->posts} 
			WHERE post_content LIKE '%[entusiastics-quiz-game %]%' AND post_title!=''
			AND post_status='publish'");	

		// match posts to exams
		foreach($quizzes as $cnt => $quiz) {

			foreach($posts as $post) {

				if(strstr($post->post_content, "[entusiastics-quiz-game ".$quiz->id."]")) {

					$quizzes[$cnt]->post = $post;			
					break;
					
				}

			}
		}

		return $quizzes;

	} // end getQuizById()getQuizzesByCategoryId

	public function checkIfPublished($id) {

		global $wpdb;

		$sql =  $wpdb->prepare("SELECT ID FROM {$wpdb->posts} WHERE post_content LIKE '%[entusiastics-quiz-game %d]%' AND post_title!='' AND post_status='publish'", $id);
		$post_id = $wpdb->get_var($sql);

		return $post_id;

	} // end getQuizById()

	public function checkIfPublishedUpdated($id, $post_status) {
		global $wpdb;

		$sql =  $wpdb->prepare("SELECT ID FROM {$wpdb->posts} WHERE post_content LIKE '%[entusiastics-quiz-game %d]%' AND post_title!='' AND post_status=%s", $id, $post_status);
		$post_id = $wpdb->get_var($sql);

		return $post_id;

	} // end getQuizById()

	public function updateQuizCategoryToNonCategorized($category_id) {

		global $wpdb;

		$quizzes_updated = $wpdb->query($wpdb->prepare("UPDATE ".ENTUSIASTICS_QUIZ_GAME_QUIZZES." SET
			category_id=1 WHERE category_id=%d", $category_id));

		return $quizzes_updated;
		
	} // end getQuizById()

	public function add($vars) {

		global $wpdb;
		
		$quiz_title = sanitize_text_field($vars['title']);
		$intro_text = sanitize_text_field($vars['intro_text']);
		$description_text = sanitize_text_field($vars['description_text']);
		$thumbnail_summary = sanitize_text_field($vars['thumbnail_summary']);
		$category_id = empty($vars['category_id']) ? 1 : $vars['category_id'];
		$duration = empty($vars['duration']) ? null : $vars['duration'];
		$image_url = empty($vars['image_url']) ? null : $vars['image_url'];
		$image_id = empty($vars['image_id']) ? null : intval($vars['image_id']);
		$theme_color = empty($vars['theme_color']) ? '#d2d2d2' : sanitize_text_field($vars['theme_color']);
		$publish_state = $vars['publish_state'];
		$comments = empty($vars['comments']) ? 0 : 1;
		
		$result = $wpdb->query($wpdb->prepare("INSERT INTO ".ENTUSIASTICS_QUIZ_GAME_QUIZZES." SET publish_state=%d, title=%s, intro_text=%s, description_text=%s, category_id=%d, duration=%s, image_url=%s, image_id=%s, thumbnail_summary=%s, theme_color=%s, comments=%d, created_at=NOW(), updated_at=NOW()", $publish_state, $quiz_title, $intro_text, $description_text, intval($category_id), $duration, $image_url, $image_id, $thumbnail_summary, $theme_color, $comments));

		if($result === false) throw new Exception(__('DB Error', 'entusiastics-quiz-game'));

		$quiz_id = $wpdb->insert_id;

		if(isset($publish_state)) {
			$this->manage_post_attached_to_quizz($quiz_id, $vars, $publish_state, '0', 'create', $comments);
		}		

		return $quiz_id;

	} // end add
	
	function save($vars, $id) {

		global $wpdb;
		
		$id = intval($id);
		
		$quiz_title = sanitize_text_field($vars['title']);
		$intro_text = sanitize_text_field($vars['intro_text']);
		$description_text = sanitize_text_field($vars['description_text']);
		$thumbnail_summary = sanitize_text_field($vars['thumbnail_summary']);
		$category_id = empty($vars['category_id']) ? 1 : $vars['category_id'];
		$duration = empty($vars['duration']) ? null : $vars['duration'];
		$image_url = empty($vars['image_url']) ? null : $vars['image_url'];
		$image_id = empty($vars['image_id']) ? null : intval($vars['image_id']);
		$theme_color = empty($vars['theme_color']) ? '#d2d2d2' : sanitize_text_field($vars['theme_color']);
		$publish_state = $vars['publish_state'];
		$comments = empty($vars['comments']) ? 0 : 1;
		$old_publish_state = $vars['old_publish_state'];
		
		$result = $wpdb->query($wpdb->prepare("UPDATE ".ENTUSIASTICS_QUIZ_GAME_QUIZZES." SET
			publish_state=%s, title=%s, intro_text=%s, description_text=%s, category_id=%d, duration=%s, image_url=%s, image_id=%s, thumbnail_summary=%s, theme_color=%s, comments=%d, updated_at=NOW() WHERE id=%d", 
			$publish_state, $quiz_title, $intro_text, $description_text, $category_id, $duration, $image_url, $image_id, $thumbnail_summary, $theme_color, $comments, $id));

		if($result === false) throw new Exception(__('DB Error', 'entusiastics-quiz-game'));
		
		if(isset($publish_state)) {
			$this->manage_post_attached_to_quizz($id, $vars, $publish_state, $old_publish_state, 'update', $comments);
		}

		return true;	
	}
	
	function delete($id) {
		
		$this->remove($id);
		
		entusiastics_quiz_game_redirect("admin.php?page=entusiastics-quiz-game-quizzes-list");
	}

	function bulkDelete($items) {

		foreach ($items as $id) {
			$this->remove($id);
		}

		entusiastics_quiz_game_redirect("admin.php?page=entusiastics-quiz-game-quizzes-list");
	}

	function remove($id) {

		global $wpdb;
		
		$id = intval($id);
		
		// delete questions
		$wpdb->query($wpdb->prepare("DELETE FROM ".ENTUSIASTICS_QUIZ_GAME_QUESTIONS." WHERE quiz_id=%d", $id));

		// delete objectives
		$wpdb->query($wpdb->prepare("DELETE FROM ".ENTUSIASTICS_QUIZ_GAME_OBJECTIVES." WHERE quiz_id=%d", $id));
		
		// delete choices
		$wpdb->query($wpdb->prepare("DELETE FROM ".ENTUSIASTICS_QUIZ_GAME_CHOICES." WHERE quiz_id=%d", $id));
		
		// delete completed records
		$wpdb->query($wpdb->prepare("DELETE FROM ".ENTUSIASTICS_QUIZ_GAME_COMPLETED." WHERE quiz_id=%d", $id));
		
		// delete the quiz
		$wpdb->query($wpdb->prepare("DELETE FROM ".ENTUSIASTICS_QUIZ_GAME_QUIZZES." WHERE id=%d", $id));
		
		// delete the page
		$this->manage_post_attached_to_quizz($id, $vars = [], '0', '0','delete', "");
	}

	function manage_post_attached_to_quizz($quiz_id, $vars, $publish_state, $old_publish_state, $action, $comments) {

		$state = 'draft';
		$comment_status = 'closed';
		$post = array();

		switch ($publish_state) {
			case '0':
			$state = 'draft';
			break;
			case '1':
			$state = 'private';
			break;
			case '2':
			$state = 'publish';
			break;
			default:
			$state = 'draft';
			break;
		}

		switch ($comments) {
			case '0':
			$comment_status = 'closed';
			break;
			case '1':
			$comment_status = 'open';
			break;
			default:
			$comment_status = 'closed';
			break;
		}

		if($action == 'create') {

			$post = array(
				'post_content' => '[entusiastics-quiz-game '.$quiz_id.']', 
				'post_name' => sanitize_text_field($vars['title']), 
				'post_title' => sanitize_text_field($vars['title']),
				'comment_status' => $comment_status,
				'post_status' => $state,
				'post_type' =>'post'
			);

			wp_insert_post($post);

		} 

		if ($action == 'update') {

			switch ($old_publish_state) {
				case '0':
				$state = 'draft';
				break;
				case '1':
				$state = 'private';
				break;
				case '2':
				$state = 'publish';
				break;
				default:
				$state = 'draft';
				break;
			}

			$post_id = $this->checkIfPublishedUpdated($quiz_id, $state);

			if($post_id) {

				$post = array( 
					'ID' => $post_id,
					'post_content' => '[entusiastics-quiz-game '.$quiz_id.']', 
					'post_name'=> sanitize_text_field($vars['title']), 
					'post_title'=>sanitize_text_field($vars['title']),
					'comment_status' => $comment_status,
					'post_status'=> $state,
					'post_type'=>'post'
				);
			}

			wp_update_post($post);
		} 

		if($action == 'delete') {
			$post_id = $this->checkIfPublished($quiz_id);
			wp_delete_post($post_id, true);
		}	
	}
	
	// copy / duplicate quiz
	public function copy($id) {

		global $wpdb;

		$id = intval($id);

	   	// select & copy quiz
		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_QUIZZES." WHERE id=%d", $id);
		$quiz = $wpdb->get_row($sql, ARRAY_A);
		$copy_msg = __('(Copia)', 'entusiastics-quiz-game');
		$quiz['title'] = sprintf('%s %s', stripslashes($quiz['title']), $copy_msg);

		$_quiz = new Quiz();  
		$new_id = $_quiz->add($quiz);

	   	// select & copy results
		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_RESULTS." WHERE quiz_id=%d ORDER BY id", $id);
		$results = $wpdb->get_results($sql, ARRAY_A);

		$_result = new Result();

		foreach($results as $result) {
			$result['quiz_id'] = $new_id;
			$result['title'] = stripslashes($result['title']);
			$result['description'] = stripslashes($result['description']);
			$_result->add($result);
		}

		// copy objectives
		$sql = $wpdb->prepare("SELECT * FROM " . ENTUSIASTICS_QUIZ_GAME_OBJECTIVES . " WHERE quiz_id=%d ORDER BY id", $id);
		$objectives = $wpdb->get_results($sql, ARRAY_A);

		if(!empty($objectives)) {

			foreach($objectives as $objective) {

				$wpdb->query($wpdb->prepare("INSERT INTO " . ENTUSIASTICS_QUIZ_GAME_OBJECTIVES . " SET name=%s, description=%s, quiz_id=%d", $objective['name'], $objective['description'], $new_id));

			}

		}

	   // select & copy questions and choices
	   $id_matches = array(); // we'll use this to match old IDs with new IDs. Important because choices may contain ID of new question in "goto"
	   $sql = $wpdb->prepare("SELECT * FROM " . ENTUSIASTICS_QUIZ_GAME_QUESTIONS . " WHERE quiz_id=%d ORDER BY id", $id);
	   $questions = $wpdb->get_results($sql, ARRAY_A);

	   $_question = new Question();

	   foreach($questions as $cnt => $question) {
	   	$question['title'] = stripslashes($question['title']);
	   	$question['question'] = stripslashes($question['question']);
	   	$question['quiz_id'] = $new_id;
	   	$new_q_id = $_question->add($question, $new_id);
	   	$id_matches[$question['id']] = $new_q_id;
	   	$questions[$cnt]['new_id'] = $new_q_id;
	   }
	   
	   // now transfer choices
	   foreach($questions as $question) {

	   	$sql = $wpdb->prepare("SELECT * FROM ". ENTUSIASTICS_QUIZ_GAME_CHOICES." WHERE question_id=%d ORDER BY id", $question['id']);
	   	$choices = $wpdb->get_results($sql);

	   	foreach($choices as $choice) {

	   		$choice->choice = stripslashes($choice->choice);
	   		$choice->question_id = $question['new_id'];
	   		if(is_numeric($choice->goto)) $choice->goto = @$id_matches[$choice->goto];

	   		$wpdb->query($wpdb->prepare("INSERT INTO " . ENTUSIASTICS_QUIZ_GAME_CHOICES . " SET quiz_id=%d, question_id=%d, choice=%s, goto=%s",
	   			$new_id, $choice->question_id, $choice->choice, $choice->goto));
	   		
	      } // end foreach choice
	  }
	} // end copy()

}