<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://entusiastics.com
 * @since      1.0.0
 *
 * @package    Entusiastics_Quiz_Game
 * @subpackage Entusiastics_Quiz_Game/public
 */

class Entusiastics_Quiz_Game_Template_Functions {

	/**
	 * Private static reference to this class
	 * Useful for removing actions declared here.
	 *
	 * @var 	object 		$_this
 	 */
	private static $_this;

	/**
	 * The post meta data
	 *
	 * @since 		1.0.0
	 * @access 		private
	 * @var 		string 			$meta    			The post meta data.
	 */
	private $meta;

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version 			The version of this plugin.
	 */
	public function __construct($plugin_name, $version) {

		self::$_this = $this;

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	} // __construct()

	/**
	 * Includes the quizz main template
	 *
	 * @hooked 		entusiastics-quiz-game-loop-content 		10
	 *
	 * @param 		string 		$name 		A Template Name
	 * @param 		array 		$args 		Params passed to Template
	 */
	public function entusiastics_quiz_template($name, $args) {

		include entusiastics_quiz_game_get_template($name);

	} // entusiastics_quiz_template()

	/**
	 * Includes the quizz frontpage template
	 *
	 * @hooked 		entusiastics-quiz-game-loop-content 		10
	 *
	 * @param 		string 		$name 		A Template Name
	 * @param 		array 		$args 		Params passed to Template
	 */
	public function entusiastics_question_template($name, $args) {

		include entusiastics_quiz_game_get_template($name);

	} // entusiastics_question_template()

	/**
	 * Includes the quizz frontpage template
	 *
	 * @hooked 		entusiastics-quiz-game-loop-content 		10
	 *
	 * @param 		string 		$name 		A Template Name
	 * @param 		array 		$args 		Params passed to Template
	 */
	public function entusiastics_video_template($name, $args) {

		include entusiastics_quiz_game_get_template($name);

	} // entusiastics_question_template()


	/**
	 * Includes the quizz frontpage template
	 *
	 * @hooked 		entusiastics-quiz-game-loop-content 		10
	 *
	 * @param 		string 		$name 		A Template Name
	 * @param 		array 		$args 		Params passed to Template
	 */
	public function entusiastics_latest_quizzes_template($name, $args) {

		global $wpdb;

		// if only have one Quizz
		if(count($args) == 1) { 

			$sql = $wpdb->prepare("SELECT * FROM {$wpdb->posts} WHERE post_content LIKE '%[entusiastics-quiz-game %d]%' AND post_status='publish'", $args[0]->id);
			$args[0]->post = $wpdb->get_row($sql);

		} else {

			$shortcode = '%['.$this->plugin_name.' %]%';

			// now select all posts that have entusiastics shortcode in them
			$sql = $wpdb->prepare("SELECT * FROM {$wpdb->posts} WHERE post_content LIKE %s AND post_status='publish' ORDER BY post_date DESC", $shortcode);
			$args['posts'] = $wpdb->get_results($sql);

			foreach($args as $cnt => $quiz) {

				foreach($args['posts'] as $post) {

					if(strstr($post->post_content, "[".$this->plugin_name." ". @$quiz->id."]")) {

						$args[$cnt]->post = $post;			
						break;

					}

				}
			}
		}

		include entusiastics_quiz_game_get_template($name);

	} // entusiastics_latest_quizzes_template()

	/**
	 * Includes the quizz result template
	 *
	 * @hooked 		entusiastics-quiz-game-loop-content 		15
	 *
	 * @param 		name 		$name 		A Template Name
	 * @param 		array 		$args 		Params passed to Template
	 */
	public function entusiastics_result_template($name, $args) {

		include entusiastics_quiz_game_get_template($name);

	} // entusiastics_result_template()

	/**
	 * Includes the not found quizzes template
	 *
	 * @hooked 		entusiastics-quiz-game-loop-content 		15
	 *
	 * @param 		name 		$name 		A Template Name
	 * @param 		array 		$args 		Params passed to Template
	 */
	public function entusiastics_not_found_template($name, $args) {

		include entusiastics_quiz_game_get_template($name);

	} // entusiastics_not_found_template()


	/**
	 * Returns a reference to this class. Used for removing
	 * actions and/or filters declared using an object of this class.
	 *
	 * @see  	http://hardcorewp.com/2012/enabling-action-and-filter-hook-removal-from-class-based-wordpress-plugins/
	 * @return 	object 		This class
	 */
	static function this() {

		return self::$_this;

	} // this()

} // class